/************************************************************************/
/*									*/
/*  This is the common header file for the RCU driver & library		*/
/*									*/
/*  15. Mar. 04  MAJO  created						*/
/*									*/
/*******C 2004 - The software with that certain something****************/

#ifndef _RCU_COMMON_H
#define _RCU_COMMON_H

#include "rcc_error.h"

/*************/
/* Constants */
/*************/
#define BULK_OUT_SIZE 1024
#define BULK_IN_SIZE  1024
#define P_ID_RCU      20   
#define P_ID_RCU_STR  "RCU library"

/*********/
/* Types */
/*********/
typedef struct 
{
  u_int nbytes;
  u_char data[BULK_OUT_SIZE];
} rcu_bulk_out_t;

typedef struct 
{
  u_int nbytes;
  u_char data[BULK_IN_SIZE];
} rcu_bulk_in_t;


typedef struct
{
  u_char bRequest;           
  u_char bmRequestType;
  u_char data[100];
  u_short wValue;
  u_short wIndex;
  u_short wLength;
} rcu_control_t;


/*************/
/*ioctl codes*/
/*************/
enum
{
  SEND_DATA = 1,
  GET_DATA,
  CONTROL
};


/*************/
/*error codes*/
/*************/
enum
{
  RCU_SUCCESS = 0,
  RCU_FILE = (P_ID_RCU <<8) + 1,
  RCU_NOTOPEN,
  RCU_NO_CODE,
  RCU_EFAULT,
  RCU_ERROR_FAIL
};


/***************/
/*error strings*/
/***************/
#define RCU_SUCCESS_STR        "Function successfully executed"
#define RCU_FILE_STR           "Failed to open / close device file"
#define RCU_NOTOPEN_STR        "The library has not yet been opened"
#define RCU_EFAULT_STR         "Error from the driver"
#define RCU_NO_CODE_STR        "Unknown error"

#ifdef __cplusplus
extern "C" {
#endif

/***********************************/ 
/*Official functions of the library*/
/***********************************/ 
u_int RCU_Open(void);
u_int RCU_Close(void);
u_int RCU_Send(rcu_bulk_out_t *out);
u_int RCU_Get(rcu_bulk_in_t *in);
u_int RCU_Control(rcu_control_t *ctrl);


/***********************************/ 
/*Internal functions of the library*/
/***********************************/ 
u_int RCU_err_get(err_pack err, err_str pid, err_str code);

#ifdef __cplusplus
}
#endif

#endif
