/************************************************************************/
/*									*/
/*  This is the RCU library 	   				        */
/*									*/
/*  16. Mar. 04  MAJO  created						*/
/*									*/
/*******C 2004 - The software with that certain something****************/

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include "rcu_lib.h"


/***********/
/* Globals */
/***********/
static int fd, rcu_is_open = 0;
u_int traceLevel, packageId;


/******************/
u_int RCU_Open(void)
/******************/
{  
  u_int ret;
  
  if (rcu_is_open)
  {
    rcu_is_open++;             /* keep track of multiple open calls */
    return(RCC_ERROR_RETURN(0, RCU_SUCCESS));
  }

  /* open the error package */
  ret = rcc_error_init((err_pid)P_ID_RCU, RCU_err_get);
  if (ret)
  {
    DEBUG_TEXT(P_ID_RCU, 5 ,("RCU_Open: Failed to open error package\n"))
    return(RCC_ERROR_RETURN(0, RCU_ERROR_FAIL)); 
  }
  DEBUG_TEXT(P_ID_RCU, 20 ,("RCU_Open: error package opened\n")) 

  DEBUG_TEXT(P_ID_RCU, 20 ,("RCU_Open: Opening /dev/usb/rcu\n"))
  if ((fd = open("/dev/usb/rcu", O_RDWR)) < 0)
  {
    perror("open");
    return(RCC_ERROR_RETURN(0, RCU_FILE)); 
  }
  DEBUG_TEXT(P_ID_RCU, 20 ,("RCU_Open: /dev/usb/rcu is open\n"))

  rcu_is_open = 1;
  return(RCU_SUCCESS);
}


/*******************/
u_int RCU_Close(void)
/*******************/
{
  RCUISOPEN
  
  if (rcu_is_open > 1)
    rcu_is_open--;
  else
  {  
    close(fd);
    rcu_is_open = 0;
  }
  
  return(RCU_SUCCESS);
}


/*********************************/
u_int RCU_Send(rcu_bulk_out_t *out)
/*********************************/
{
  int ret, e2;

  RCUISOPEN
 
  ret = ioctl(fd, SEND_DATA, out);
  e2 = errno;
  if (ret)
  {
    DEBUG_TEXT(P_ID_RCU, 5 ,("RCU_Send: Error from ioctl, ret = 0x%08x\n", -ret))
    DEBUG_TEXT(P_ID_RCU, 5 ,("RCU_Send: Error from ioctl, e2 = 0x%08x\n", e2))
    return(RCC_ERROR_RETURN(0, -ret));
  }

return(RCU_SUCCESS);
}


/******************************/
u_int RCU_Get(rcu_bulk_in_t *in)
/******************************/
{
  int ret, e2;

  if (!rcu_is_open) 
    return(RCC_ERROR_RETURN(0, RCU_NOTOPEN));

  ret = ioctl(fd, GET_DATA, in);
  e2 = errno;
  if (ret)
  {
    DEBUG_TEXT(P_ID_RCU, 5 ,("RCU_Get: Error from ioctl, ret = 0x%08x\n", ret))
    DEBUG_TEXT(P_ID_RCU, 5 ,("RCU_Get: Error from ioctl, e2 = 0x%08x\n", e2))
   return(RCC_ERROR_RETURN(0, -ret));
  }

return(RCU_SUCCESS);
}


/************************************/
u_int RCU_Control(rcu_control_t *ctrl)
/************************************/
{
  int ret;

  if (!rcu_is_open) 
    return(RCC_ERROR_RETURN(0, RCU_NOTOPEN));

  ret = ioctl(fd, CONTROL, ctrl);
  if (ret)
  {
    DEBUG_TEXT(P_ID_RCU, 5 ,("RCU_Control: Error from ioctl, errno = 0x%08x\n", errno))
    return(RCC_ERROR_RETURN(0, -ret));
  }

return(RCU_SUCCESS);
}


/***************************************************************/
unsigned int RCU_err_get(err_pack err, err_str pid, err_str code)
/***************************************************************/
{ 
  strcpy(pid, P_ID_RCU_STR);

  switch (RCC_ERROR_MINOR(err))
  {  
    case RCU_SUCCESS:    strcpy(code, RCU_SUCCESS_STR);     break;
    case RCU_FILE:       strcpy(code, RCU_FILE_STR);        break;
    case RCU_EFAULT:     strcpy(code, RCU_EFAULT_STR);      break;
    default:             strcpy(code, RCU_NO_CODE_STR);     return(RCC_ERROR_RETURN(0, RCU_NO_CODE)); break;
  }
  return(RCC_ERROR_RETURN(0, RCU_SUCCESS));
}
