/************************************************************************/
/*									*/
/*  This is the header file for the U2F library				*/
/*									*/
/*  24. Jun. 04  MAJO  created						*/
/*									*/
/*******C 2004 - The software with that certain something****************/

#ifndef _U2F_H
#define _U2F_H

#include "rcu_lib.h"

/*************/
/* Constants */
/*************/
#define P_ID_U2F     21
#define P_ID_U2F_STR "U2F library"

/* Instruction codes */
#define I_WRREG    0
#define I_RDREG    1
#define I_WRPDM    2
#define I_RDPDM    3
#define I_RDDTM    4
#define I_RDEVL    5
#define I_SWTRG    6
#define I_SWRST    7
#define I_TWRST    8



/* Addresses */
#define A_PMEM      0x0000
#define A_DMEM      0x0000
#define A_REGISTERS 0x0000


/* Registers */
#define O_BSLCT 1
#define O_TMASK 2
#define O_ACTCH 3
#define O_ZSCTL 4
#define O_ZSTHD 5
#define O_ZSPED 6
#define O_TATHD 7
#define O_TBTHD 8
#define O_NSEVT 9
#define O_TRGEV 10
#define O_PRTRG 11

/* Sizes */
#define S_DMEM 128
#define S_PMEM 512
#define S_RMEM 128
#define S_ACL  256
#define S_DM   256

/* Commands */
#define C_SWTRG     0x0006
#define C_SWRST	    0x0007	
#define C_TWRST	    0x0008

/* CHANNEL ADDRESS */
#define O_CHA     0x0000
#define O_CHB     0x0400
#define O_CHC     0x0800
	

/*********/
/* Types */
/*********/
typedef struct
{
  u_int pattern_error; /* Possible values are TRUE and FALSE */
  u_int abort;         /* Possible values are TRUE and FALSE */
  u_int timeout;       /* Possible values are TRUE and FALSE */
  u_int altro_error;   /* Possible values are TRUE and FALSE */
} errst_t;

typedef struct
{
  u_int tw;    /* Legal values are 0 - 0x3fff     */
  u_int bmd;   /* Legal values are 0 or 1         */
  u_int mode;  /* Legal values are 0, 2 or 3      */
  u_int remb;  /* Legal values are 0 - 0xf        */
  u_int empty; /* Legal values are TRUE and FALSE */
  u_int full;  /* Legal values are TRUE and FALSE */
  u_int rd_pt; /* Legal values are 0 to 7         */
  u_int wr_pt; /* Legal values are 0 to 7         */
} trcfg_t;

typedef struct
{
  u_int ntr; /* Possible values are 0 - 0xffff */
  u_int nta; /* Possible values are 0 - 0xffff */
} trcnt_t;


/*************/
/*error codes*/
/*************/
enum
{
  U2F_SUCCESS = 0,
  U2F_FILE = (P_ID_U2F<<8) + 1,
  U2F_NOTOPEN,
  U2F_RCU_FAIL,
  U2F_RANGE,
  U2F_ODD_BYTES,
  U2F_NODATA,
  U2F_NO_CODE,
  U2F_INC_BYTES,
  U2F_ERROR_FAIL
};


/***************/
/*error strings*/
/***************/
#define U2F_SUCCESS_STR        "Function successfully executed"
#define U2F_FILE_STR           "Failed to open / close data file"
#define U2F_NOTOPEN_STR        "The library has not been opened yet"
#define U2F_RCU_FAIL_STR       "Error from RCU library"
#define U2F_RANGE_STR          "A parameter is out of range"
#define U2F_ODD_BYTES_STR      "Wrong number of bytes received"
#define U2F_NO_CODE_STR        "Unknown error"
#define U2F_INC_BYTES_STR      "DML and DMH have delivered inconsistent amounts of data in a read operation"
#define U2F_NODATA_STR         "The files you are reading does not contain enought data"
 
/********/
/*Macros*/
/********/
#define U2FISOPEN {if(!u2f_is_open) return(U2F_NOTOPEN);}

#ifdef __cplusplus
extern "C" {
#endif

/***********************************/ 
/*Official functions of the library*/
/***********************************/ 
u_int U2F_Open(void);
u_int U2F_Close(void);
u_int U2F_IMEM_Read(u_int isize, u_int *osize, u_int offset, u_int data[]);
u_int U2F_PMEM_Read(u_int isize, u_int *osize, u_int offset, u_int data[]);
u_int U2F_RMEM_Read(u_int isize, u_int *osize, u_int offset, u_int data[]);
u_int U2F_ACL_Read(u_int isize, u_int *osize, u_int offset, u_int data[]);
u_int U2F_DM1_Read(u_int isize, u_int *osize, u_int offset, u_int data[]);
u_int U2F_DM2_Read(u_int isize, u_int *osize, u_int offset, u_int data[]);
u_int U2F_IMEM_Write(u_int osize, u_int offset, u_int data[]);
u_int U2F_PMEM_Write(u_int osize, u_int offset, u_int data[]);
u_int U2F_RMEM_Write(u_int osize, u_int offset, u_int data[]);
u_int U2F_ACL_Write(u_int osize, u_int offset, u_int data[]);
u_int U2F_DM1_Write(u_int osize, u_int offset, u_int data[]);
u_int U2F_DM2_Write(u_int osize, u_int offset, u_int data[]);
u_int U2F_Reg_Write(u_int reg, u_int data);
u_int U2F_Reg_Read(u_int reg, u_int *data);
u_int U2F_ERRST_decode(u_int data, errst_t *parameters);
u_int U2F_TRCFG_decode(u_int data, trcfg_t *parameters);
u_int U2F_TRCNT_decode(u_int data, trcnt_t *parameters);
u_int U2F_TRCFG_encode(u_int *data, trcfg_t parameters);
u_int U2F_File_Read(char *name, u_int size, u_int data[]);
u_int U2F_File_Write(char *name, u_int size, u_int data[]);
u_int U2F_Exec_Command(u_int command, u_int data);
u_int U2F_Reset(void);

u_int U2F_DMEM_Read(u_int isize, u_int *osize, u_int offset, u_int data[]);
u_int U2F_Evl_Read(u_int reg, u_int *data);

/***********************************/ 
/*Internal functions of the library*/
/***********************************/ 
u_int U2F_err_get(err_pack err, err_str pid, err_str code);

#ifdef __cplusplus
}
#endif

#endif
