/************************************************************************/
/*									*/
/*  This is the U2F library 	   				        */
/*									*/
/*  24. Jun. 04  MAJO  created						*/
/*									*/
/*******C 2004 - The software with that certain something****************/

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <fcntl.h>
#include "u2f.h"


/***********/
/* Globals */
/***********/
static u_int u2f_is_open = 0;
extern u_int traceLevel, packageId;


/******************/
u_int U2F_Open(void)
/******************/
{  
  u_int ret;
  
  if (u2f_is_open)
  {
    u2f_is_open++;             /* keep track of multiple open calls */
    return(RCC_ERROR_RETURN(0, U2F_SUCCESS));
  }

  /* open the error package */
  ret = rcc_error_init((err_pid)P_ID_U2F, U2F_err_get);
  if (ret)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_Open: Failed to open error package\n"))
    return(RCC_ERROR_RETURN(0, U2F_ERROR_FAIL)); 
  }
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_Open: error package opened\n")) 

  ret = RCU_Open();
  if (ret)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_Open: Failed to open the RCU library\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }
  DEBUG_TEXT(P_ID_U2F, 10 ,("U2F_Open: RCU library opened\n")) 
  
  u2f_is_open = 1;
  return(U2F_SUCCESS);
}


/*******************/
u_int U2F_Close(void)
/*******************/
{
  u_int ret;

  U2FISOPEN
  
  if (u2f_is_open > 1)
    u2f_is_open--;
  else
  {  
    RCU_Close();
    if (ret)
    {
      DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_Close: Failed to close the RCU library\n"))
      return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
    }
    DEBUG_TEXT(P_ID_U2F, 10 ,("U2F_Close: RCU library closed\n")) 
    u2f_is_open = 0;
  }
  
  return(U2F_SUCCESS);
}



/************************************************************************/
u_int U2F_PMEM_Read(u_int isize, u_int *osize, u_int offset, u_int data[])
/************************************************************************/
{
  rcu_bulk_out_t out;
  rcu_bulk_in_t in;
  u_short *msg;
  u_int *ptr, ret, msize, loop;
  
  U2FISOPEN
 
  msize = isize;  /* one word in the PMEM takes two 16 bit words in the message */
 
  /*if ((isize + offset) > S_PMEM)
  {    
    DEBUG_TEXT(P_ID_U2F, 5 ,("PMEM_Read: Parameter isize is out of range\n")) 
    return(RCC_ERROR_RETURN(0, U2F_RANGE));
  } */
 
  /* Construct a message */
  msg = (u_short *)&out.data[0];
  msg[0] = I_RDPDM; 
  msg[1] = A_PMEM + offset;
  out.nbytes = 4;

  /* Tell the USB2FEC that we want to read the PMEM  */
  ret = RCU_Send(&out);
  if (ret)
  {      
    DEBUG_TEXT(P_ID_U2F, 5 ,("PMEM_Read: Error from RCU_Send\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }    

  /* Now read the data from PMEM */
  ret = RCU_Get(&in);
  if (ret)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("PMEM_Read: Error from RCU_Get\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }

  DEBUG_TEXT(P_ID_U2F, 20 ,("PMEM_Read: %d bytes received from the USB2FEC\n", in.nbytes))
  if (in.nbytes != 2)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("PMEM_Read: Wrong number of bytes received\n"))
    *osize = 0;
    return(RCC_ERROR_RETURN(ret, U2F_ODD_BYTES)); 
  }
  
  *osize = isize;
  ptr = (u_int *) &in.data[0];
  /* copy the data from the rcu_bulk_in_t structure to the data array */
  for(loop = 0; loop < *osize; loop++)
    data[loop] = ptr[loop] & 0xffff;

  return(U2F_SUCCESS); 
}
		
  
/************************************************************************/
u_int U2F_DMEM_Read(u_int isize, u_int *osize, u_int offset, u_int data[])
/************************************************************************/
{
  rcu_bulk_out_t out;
  rcu_bulk_in_t in;
  u_short *msg;
  u_int *ptr, ret, msize, psize, loop, loop2;
  
  U2FISOPEN
 
  msize = 4*isize;  /* one word in the DMEM takes four 16 bit words in the message */
 
 /* if ((isize + offset) > S_DMEM)
  {    
    DEBUG_TEXT(P_ID_U2F, 5 ,("DMEM_Read: Parameter isize is out of range\n")) 
    return(RCC_ERROR_RETURN(0, U2F_RANGE));
  } */ 
 
  /* Construct a message */
  msg = (u_short *)&out.data[0];
  msg[0] = (isize << 5) + I_RDDTM; 
  msg[1] = A_DMEM + offset;
  out.nbytes = 4;

// debug 
printf("msg[0] = %x\n", msg[0]);
printf("msg[1] = %x\n", msg[1]);

  /* Tell the USB2FEC that we want to read the PMEM  */
  ret = RCU_Send(&out);
  if (ret)
  {      
    DEBUG_TEXT(P_ID_U2F, 5 ,("PMEM_Read: Error from RCU_Send\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }    

  /* Now read the data from PMEM */
  ret = RCU_Get(&in);
  if (ret)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("DMEM_Read: Error from RCU_Get\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }


//debug 
printf("nr. of received bytes = %d\n", in.nbytes);

  DEBUG_TEXT(P_ID_U2F, 20 ,("DMEM_Read: %d bytes received from the USB2FEC\n", in.nbytes))
  if (in.nbytes != (2*msize))
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("DMEM_Read: Wrong number of bytes received\n"))
    *osize = 0;
    return(RCC_ERROR_RETURN(ret, U2F_ODD_BYTES)); 
  }
  
  *osize = msize;
  psize = 2*isize;
  ptr = (u_int *) &in.data[0];
  /* copy the data from the rcu_bulk_in_t structure to the data array */
  for(loop = 0; loop < psize; loop++)
    {
    data[2*loop] =     ptr[loop] & 0xffff;
    data[2*loop+1] =   (ptr[loop] >> 16) & 0xffff;  
    }
  return(U2F_SUCCESS); 
}
		
  
    
/************************************************************/
u_int U2F_PMEM_Write (u_int osize, u_int offset, u_int data[])
/************************************************************/
{
  rcu_bulk_out_t out;
  u_short *msg, ptr;
  u_int ret, msize, loop;
  
  U2FISOPEN
 
  msize = osize;  /* one word in the PMEM takes two 16 bit words in the message */
 
  if ((osize + offset) > S_PMEM)
  {    
    DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_PMEM_Write: Parameter isize is out of range\n")) 
    return(RCC_ERROR_RETURN(0, U2F_RANGE));
  } 
 
  /* Construct a message */
  msg = (u_short *)&out.data[0];
  msg[0] = I_WRPDM; 
  msg[1] = A_PMEM + offset;

  ptr = 2;
  for(loop = 0; loop < osize; loop++)
  {
    msg[ptr++] = data[loop] & 0x03ff;
    msg[ptr++] = 0;
  } 
  out.nbytes = (2 + msize) * 2;  /*  16 bit word to byte */

  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[0]=0x%04x\n", msg[0])) 
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[1]=0x%04x\n", msg[1])) 
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[2]=0x%04x\n", msg[2])) 
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[3]=0x%04x\n", msg[3])) 
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[4]=0x%04x\n", msg[4])) 
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[5]=0x%04x\n", msg[5])) 
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[6]=0x%04x\n", msg[6])) 
  DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_PMEM_Write: msg[7]=0x%04x\n", msg[7])) 

  /* Tell the USB2FEC that we want to write the PMEM  */
  ret = RCU_Send(&out);
  if (ret)
  {      
    DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_PMEM_Write: Error from RCU_Send\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }    

  return(U2F_SUCCESS);   
}

  
  
/****************************************/
u_int U2F_Reg_Write(u_int reg, u_int data)
/****************************************/
{
  int ret;
  rcu_bulk_out_t out;
  u_short *msg;

  DEBUG_TEXT(P_ID_U2F, 20 ,("Reg_Write: reg = %d  data = %d\n", reg, data))

  U2FISOPEN
  
  if ((reg != O_BSLCT) && (reg != O_TMASK) && (reg != O_ACTCH) && (reg != O_ZSCTL) && (reg != O_ZSTHD) 
       && (reg != O_ZSPED) && (reg != O_TATHD) && (reg != O_TBTHD) && (reg != O_NSEVT) && (reg != O_PRTRG))
  {    
    DEBUG_TEXT(P_ID_U2F, 5 ,("Reg_Write: The parameter reg is out of range (reg = %d)\n", reg))
    return(RCC_ERROR_RETURN(0, U2F_RANGE)); 
  } 
  
  /* Assemble the message */
  msg = (u_short *)&out.data[0];
  msg[0] = I_WRREG; 
  msg[1] = A_REGISTERS + reg;
  msg[2] = data & 0xffff;
  out.nbytes = 6;

  /* Tell the USB2FEC that we want to write a register  */
  ret = RCU_Send(&out);
  if (ret)
  {      
    DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_Reg_Write: Error from RCU_Send\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }    
  
  return(U2F_SUCCESS);
}

  
/****************************************/
u_int U2F_Reg_Read(u_int reg, u_int *data)
/****************************************/
{
  int ret;
  rcu_bulk_out_t out;
  rcu_bulk_in_t in;
  u_short *msg;
  
  U2FISOPEN
 
  if ((reg != O_BSLCT) && (reg != O_TMASK) && (reg != O_ACTCH) && (reg != O_ZSCTL) && (reg != O_ZSTHD) 
       && (reg != O_ZSPED) && (reg != O_TATHD) && (reg != O_TBTHD) && (reg != O_NSEVT) && (reg != O_TRGEV) && (reg != O_PRTRG))
  {    
    DEBUG_TEXT(P_ID_U2F, 5 ,("Reg_Read: The parameter reg is out of range\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RANGE)); 
  }

  /* Assemble the message */
  msg = (u_short *)&out.data[0];
  msg[0] = I_RDREG; 
  msg[1] = A_REGISTERS + reg;
  out.nbytes = 4;
  
  /* Tell the USB2FEC that we want to read a register  */
  ret = RCU_Send(&out);
  if (ret)
  {      
    DEBUG_TEXT(P_ID_U2F, 5 ,("Reg_Read: Error from RCU_Send\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }    

  /* Now read the data from IMEM */
  ret = RCU_Get(&in);
  if (ret)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("Reg_Read: Error from RCU_Get\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }

  DEBUG_TEXT(P_ID_U2F, 20 ,("Reg_Read: %d bytes received from the USB2FEC\n", in.nbytes))
  if (in.nbytes != 2)
  {
     DEBUG_TEXT(P_ID_U2F, 5 ,("Reg_Read: Wrong number of bytes received\n"))
     return(RCC_ERROR_RETURN(ret, U2F_ODD_BYTES)); 
  }

  *data = in.data[0] + (in.data[1] << 8);

  return(U2F_SUCCESS);
}  


/****************************************/
u_int U2F_Evl_Read(u_int reg, u_int *data)
/****************************************/
{
  int ret;
  rcu_bulk_out_t out;
  rcu_bulk_in_t in;
  u_short *msg;
  
  U2FISOPEN
 
  if ((reg != O_CHA) && (reg != O_CHB) && (reg != O_CHC))
  {    
    DEBUG_TEXT(P_ID_U2F, 5 ,("Evl_Read: The channel address is out of range\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RANGE)); 
  }

  /* Assemble the message */
  msg = (u_short *)&out.data[0];
  msg[0] = I_RDEVL; 
  msg[1] = reg;
  out.nbytes = 4;
  
  /* Tell the USB2FEC that we want to read a register  */
  ret = RCU_Send(&out);
  if (ret)
  {      
    DEBUG_TEXT(P_ID_U2F, 5 ,("Evl_Read: Error from RCU_Send\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }    

  /* Now read the data from IMEM */
  ret = RCU_Get(&in);
  if (ret)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("Evl_Read: Error from RCU_Get\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }

  DEBUG_TEXT(P_ID_U2F, 20 ,("Evl_Read: %d bytes received from the USB2FEC\n", in.nbytes))
  if (in.nbytes != 2)
  {
     DEBUG_TEXT(P_ID_U2F, 5 ,("Evl_Read: Wrong number of bytes received\n"))
     return(RCC_ERROR_RETURN(ret, U2F_ODD_BYTES)); 
  }

  *data = in.data[0] + (in.data[1] << 8);

  return(U2F_SUCCESS);
}  
  

  
/*******************************************************/
u_int U2F_File_Read(char *name, u_int size, u_int data[])
/*******************************************************/
{
  u_int loop;
  FILE *inf;
  
  U2FISOPEN
  
  /* open the input file */
  inf = fopen(name, "r");
  if (inf == 0)
  {
    DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_File_Read:  Can't open input file\n")) 
    return(RCC_ERROR_RETURN(0, U2F_FILE));
  }

  /* read  the file */
  for (loop = 0; loop < size; loop++)
  {
    fscanf(inf,"%x",&data[loop]);
    if (feof(inf))
    {  
      DEBUG_TEXT(P_ID_U2F, 20 ,("U2F_File_Read: The file is too short\n")) 
      return(RCC_ERROR_RETURN(0, U2F_NODATA));
    }
  }
  fclose(inf);
  return(U2F_SUCCESS);
}

  
/********************************************************/
u_int U2F_File_Write(char *name, u_int size, u_int data[])
/********************************************************/
{
  u_int loop;
  FILE *outf;

  U2FISOPEN

  outf = fopen(name, "a+");
  if (outf==0)
  {
    DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_File_Write:  Can't open output file\n")) 
    return(RCC_ERROR_RETURN(0, U2F_FILE));
  }
  
  /* write  the file */
  for (loop = 0; loop < size; loop++)
    fprintf(outf,"%x\n",data[size]);

  fclose(outf);
  return(U2F_SUCCESS);  
}


/***********************************************/
u_int U2F_Exec_Command(u_int command, u_int data)
/***********************************************/
/*********** Modified by A. Junique ************/
{
  rcu_bulk_out_t out;
  u_short *msg;
  u_int address, ret;
  
  U2FISOPEN

  if ((command != C_SWTRG) && (command != C_SWRST) && (command != C_TWRST))
  {    
    DEBUG_TEXT(P_ID_U2F, 5 ,("Exec_Command:  The parameter <command> is out of range\n")) 
    return(RCC_ERROR_RETURN(0, U2F_RANGE));
  } 

  address = command;
    
  /* Construct a message */
  msg = (u_short *)&out.data[0];
  msg[0] = command; 
  out.nbytes = 2;
   
  /* Send the message  */
  ret = RCU_Send(&out);
  if (ret)
  {      
    DEBUG_TEXT(P_ID_U2F, 5 ,("U2F_Exec_Command: Failed to close the RCU library\n"))
    return(RCC_ERROR_RETURN(ret, U2F_RCU_FAIL)); 
  }      

  return(U2F_SUCCESS);  
}


/***************************************************************/
unsigned int U2F_err_get(err_pack err, err_str pid, err_str code)
/***************************************************************/
{ 
  strcpy(pid, P_ID_RCU_STR);

  switch (RCC_ERROR_MINOR(err))
  {  
    case U2F_SUCCESS:    strcpy(code, U2F_SUCCESS_STR);     break;
    case U2F_FILE:       strcpy(code, U2F_FILE_STR);        break;
    case U2F_RCU_FAIL:   strcpy(code, U2F_RCU_FAIL_STR);    break;
    case U2F_RANGE:      strcpy(code, U2F_RANGE_STR);       break;
    case U2F_ODD_BYTES:  strcpy(code, U2F_ODD_BYTES_STR);   break;
    case U2F_NODATA:     strcpy(code, U2F_NODATA_STR);      break;
    case U2F_INC_BYTES:  strcpy(code, U2F_INC_BYTES_STR);   break;
    default:             strcpy(code, U2F_NO_CODE_STR);     return(RCC_ERROR_RETURN(0, U2F_NO_CODE)); break;
  }
  return(RCC_ERROR_RETURN(0, U2F_SUCCESS));
}




